package com.jqtx.windows.command;


import com.alibaba.fastjson.JSONObject;
import com.jqtx.windows.common.factory.AbstractCommand;
import com.jqtx.windows.common.factory.RuntimeContextHolder;
import com.jqtx.windows.component.BackedDetailComponent;
import com.jqtx.windows.component.LoanComponent;
import com.jqtx.windows.component.LoanPlanComponent;
import com.jqtx.windows.component.LoanRepayComponent;
import com.jqtx.windows.component.model.BackedDetailModel;
import com.jqtx.windows.component.model.LoanModel;
import com.jqtx.windows.component.model.LoanPlanModel;
import com.jqtx.windows.component.model.RepayModel;
import com.jqtx.windows.infrastructure.feign.enums.DebtInfoStatusEnum;
import com.jqtx.windows.infrastructure.feign.enums.DebtPlanStatusEnum;
import com.jqtx.windows.web.request.BackedDetail;
import com.jqtx.windows.web.request.BackedRequest;
import com.jqtx.windows.web.response.JsonResult;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class BackedCommand extends AbstractCommand {

    @Autowired
    private LoanRepayComponent loanRepayComponent;

    @Autowired
    private LoanPlanComponent loanPlanComponent;

    @Autowired
    private LoanComponent loanComponent;

    @Autowired
    private BackedDetailComponent backedDetailComponent;

    @Transactional
    @Override
    public JsonResult execute() {
        String request = RuntimeContextHolder.currentRuntimeContext().getRequestBody();
        BackedRequest backedRequest = JSONObject.parseObject(request, BackedRequest.class);
        //生成 windows_repay 信息
        creditRepay(backedRequest);
        //生成 windows_backed_detail 信息
        //修改 windows_plan backAmt 信息
        //修改 windows_plan 还款状态
        //修改 windows_loan 还款状态
        backedDetail(backedRequest.getLoanNo(), new BigDecimal(backedRequest.getBackAmt()), backedRequest.getBackedNo(), backedRequest);
        return JsonResult.success();
    }

    public void creditRepay(BackedRequest backedRequest) {
        RepayModel windowsRepay = new RepayModel();
        windowsRepay.setLoanNo(backedRequest.getLoanNo());
        windowsRepay.setBackedNo(backedRequest.getBackedNo());
        windowsRepay.setBankAccount(backedRequest.getBankAccount());
        windowsRepay.setBackedAmt(new BigDecimal(backedRequest.getBackAmt()));
        windowsRepay.setIsSettle(Integer.parseInt(backedRequest.getIsSettle()));
        windowsRepay.setBackedType(backedRequest.getBackWay());
        windowsRepay.setRepayResult("SUCCESS");
        windowsRepay.setRepayStatus("FINISHED");
        windowsRepay.setCreator("浙农");
        windowsRepay.setGmtCreated(LocalDateTime.now());
        windowsRepay.setGmtModified(LocalDateTime.now());
        loanRepayComponent.insert(windowsRepay);
    }


    @Transactional
    public String backedDetail(String loanNo, BigDecimal backedAmt, String backedNo, BackedRequest backedRequest) {
        List<LoanPlanModel> planList = loanPlanComponent.getByLoan(loanNo);

        ArrayList<BackedDetail> list = backedRequest.getList();
        for (BackedDetail backedDetail : list) {
            for (LoanPlanModel loanPlanModel : planList) {
                if (loanPlanModel.getPlanIndex().toString().equals(backedDetail.getPlanIndex()) && backedAmt.compareTo(BigDecimal.ZERO) > 0) {
                    backedPlan(loanPlanModel, backedDetail, backedNo, backedRequest);
                }
            }
        }

        planList = loanPlanComponent.getByLoan(loanNo);
        LoanModel LoanModel = loanComponent.getByLoanNo(loanNo);

        List<LoanPlanModel> overduedList = planList.stream().filter(i -> i.getStatus() == DebtPlanStatusEnum.OVERDUED.getCode()).collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(overduedList)) {
            LoanModel.setLoanStatus(DebtInfoStatusEnum.OVERDUED.getCode());
        } else {
            //存在还款中计划改为还款中
            List<LoanPlanModel> backingList = planList.stream().filter(i -> i.getStatus() == DebtPlanStatusEnum.BACKING.getCode()).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(backingList)) {
                LoanModel.setLoanStatus(DebtInfoStatusEnum.SETTLED.getCode());
            }
        }
        //修改 windows_loan 还款状态
        loanComponent.updateByLoanNo(LoanModel);
        return LoanModel.getLoanStatus();
    }


    private void backedPlan(LoanPlanModel debtPlanModel, BackedDetail backedDetail, String backedNo, BackedRequest backedRequest) {
        if (debtPlanModel.getStatus() == DebtPlanStatusEnum.SETTLED.getCode()) {
            return;
        }
        BackedDetailModel debtBackedModel = new BackedDetailModel();
        debtBackedModel.setBackedType(backedRequest.getBackWay());
        debtBackedModel.setBackedNo(backedNo);
        debtBackedModel.setPlanNo(debtPlanModel.getPlanNo());
        debtBackedModel.setBackedDate(LocalDate.now());
        debtBackedModel.setBackedFa(BigDecimal.ZERO);
        debtBackedModel.setBackedFu(BigDecimal.ZERO);
        debtBackedModel.setBackedXi(BigDecimal.ZERO);
        debtBackedModel.setBackedBen(BigDecimal.ZERO);
        debtBackedModel.setBackedBurden(BigDecimal.ZERO);
        debtBackedModel.setBackedSpeaccount(BigDecimal.ZERO);
        /*if (debtPlanModel.getStatus() == DebtPlanStatusEnum.OVERDUED.getCode()) {
            //逾期的
            debtBackedModel.setBackedType("COLLECT");
        } else {
            debtBackedModel.setBackedType("RAMIND");
        }*/
        if (new BigDecimal(backedDetail.getBackedFa()).compareTo(BigDecimal.ZERO) > 0) {
            backedFa(new BigDecimal(backedDetail.getBackedFa()), debtBackedModel, debtPlanModel);
        }
        if (new BigDecimal(backedDetail.getBackedFu()).compareTo(BigDecimal.ZERO) > 0) {
            backedFu(new BigDecimal(backedDetail.getBackedFu()), debtBackedModel, debtPlanModel);
        }
        if (new BigDecimal(backedDetail.getBackedBurden()).compareTo(BigDecimal.ZERO) > 0) {
            backedBurden(new BigDecimal(backedDetail.getBackedBurden()), debtBackedModel, debtPlanModel);
        }
        if (new BigDecimal(backedDetail.getBackedXi()).compareTo(BigDecimal.ZERO) > 0) {
            backedXi(new BigDecimal(backedDetail.getBackedXi()), debtBackedModel, debtPlanModel);
        }
        if (new BigDecimal(backedDetail.getBackedBen()).compareTo(BigDecimal.ZERO) > 0) {
            backedBen(new BigDecimal(backedDetail.getBackedBen()), debtBackedModel, debtPlanModel);
        }
        debtBackedModel.setBackedAmt(debtBackedModel.getBackedFu().
                add(debtBackedModel.getBackedXi()).
                add(debtBackedModel.getBackedBen()).
                add(debtBackedModel.getBackedFa()).
                add(debtBackedModel.getBackedBurden()).
                add(debtBackedModel.getBackedSpeaccount())
        );
        //修改还款计划数据
        debtPlanModel.setBackedFu(debtPlanModel.getBackedFu().add(debtBackedModel.getBackedFu()));
        debtPlanModel.setBackedXi(debtPlanModel.getBackedXi().add(debtBackedModel.getBackedXi()));
        debtPlanModel.setBackedBen(debtPlanModel.getBackedBen().add(debtBackedModel.getBackedBen()));
        debtPlanModel.setBackedFa(debtPlanModel.getBackedFa().add(debtBackedModel.getBackedFa()));
        debtPlanModel.setBackedBurden(debtPlanModel.getBackedBurden().add(debtBackedModel.getBackedBurden()));
        debtPlanModel.setBackedSpeaccount(debtPlanModel.getBackedSpeaccount().add(debtBackedModel.getBackedSpeaccount()));
        if (debtPlanModel.getBackedBen().compareTo(debtPlanModel.getPlanBen()) >= 0) {
            if (debtPlanModel.getBackedXi().compareTo(debtPlanModel.getPlanXi()) >= 0) {
                if (debtPlanModel.getBackedFu().compareTo(debtPlanModel.getPlanFu()) >= 0) {
                    if (debtPlanModel.getBackedFa().compareTo(debtPlanModel.getPlanFa()) >= 0) {
                        if (debtPlanModel.getBackedBurden().compareTo(debtPlanModel.getPlanBurden()) >= 0) {
                            if (debtPlanModel.getBackedSpeaccount().compareTo(debtPlanModel.getPlanSpeaccount()) >= 0) {
                                debtPlanModel.setStatus(DebtPlanStatusEnum.SETTLED.getCode());
                            }
                        }
                    }
                }
            }
        }
        if (debtBackedModel.getBackedAmt().compareTo(BigDecimal.ZERO) > 0) {
            //生成 windows_backed_detail 信息
            backedDetailComponent.insert(debtBackedModel);
            //修改 windows_plan backAmt 信息
            //修改 windows_plan 还款状态
            loanPlanComponent.updateByLoanPlanModel(debtPlanModel);
        }
    }


    //还罚息
    private BigDecimal backedFa(BigDecimal backedAmt, BackedDetailModel debtBackedModel, LoanPlanModel debtPlanModel) {
        //剩余罚息
        BigDecimal remainFa = debtPlanModel.getPlanFa().subtract(debtPlanModel.getBackedFa());
        BigDecimal remainAmt = backedAmt.subtract(remainFa);
        if (remainAmt.compareTo(BigDecimal.ZERO) >= 0) {
            debtBackedModel.setBackedFa(remainFa);
        } else {
            debtBackedModel.setBackedFa(backedAmt);
        }
        return remainAmt;
    }

    //还服务费
    private BigDecimal backedFu(BigDecimal backedAmt, BackedDetailModel debtBackedModel, LoanPlanModel debtPlanModel) {
        //剩余罚息
        BigDecimal remainFu = debtPlanModel.getPlanFu().subtract(debtPlanModel.getBackedFu());
        BigDecimal remainAmt = backedAmt.subtract(remainFu);
        if (remainAmt.compareTo(BigDecimal.ZERO) >= 0) {
            debtBackedModel.setBackedFu(remainFu);
        } else {
            debtBackedModel.setBackedFu(backedAmt);
        }
        return remainAmt;
    }


    //还利息
    private BigDecimal backedXi(BigDecimal backedAmt, BackedDetailModel debtBackedModel, LoanPlanModel debtPlanModel) {
        BigDecimal remainXi = debtPlanModel.getPlanXi().subtract(debtPlanModel.getBackedXi());
        BigDecimal remainAmt = backedAmt.subtract(remainXi);
        if (remainAmt.compareTo(BigDecimal.ZERO) >= 0) {
            debtBackedModel.setBackedXi(remainXi);
        } else {
            debtBackedModel.setBackedXi(backedAmt);
        }
        return remainAmt;
    }

    //还本金
    private BigDecimal backedBen(BigDecimal backedAmt, BackedDetailModel debtBackedModel, LoanPlanModel debtPlanModel) {
        BigDecimal remainBen = debtPlanModel.getPlanBen().subtract(debtPlanModel.getBackedBen());
        BigDecimal remainAmt = backedAmt.subtract(remainBen);
        if (remainAmt.compareTo(BigDecimal.ZERO) >= 0) {
            debtBackedModel.setBackedBen(remainBen);
        } else {
            debtBackedModel.setBackedBen(backedAmt);
        }
        return remainAmt;
    }

    //还融担成本
    private BigDecimal backedBurden(BigDecimal backedAmt, BackedDetailModel debtBackedModel, LoanPlanModel debtPlanModel) {
        //剩余融担成本
        BigDecimal remainBurden = debtPlanModel.getPlanBurden().subtract(debtPlanModel.getBackedBurden());
        BigDecimal remainAmt = backedAmt.subtract(remainBurden);
        if (remainAmt.compareTo(BigDecimal.ZERO) >= 0) {
            debtBackedModel.setBackedBurden(remainBurden);
        } else {
            debtBackedModel.setBackedBurden(backedAmt);
        }
        return remainAmt;
    }

    //还资产分账
    private BigDecimal backedSpeaccount(BigDecimal backedAmt, BackedDetailModel debtBackedModel, LoanPlanModel debtPlanModel) {
        //剩余资产分账
        BigDecimal remainSpeaccount = debtPlanModel.getPlanSpeaccount().subtract(debtPlanModel.getBackedSpeaccount());
        BigDecimal remainAmt = backedAmt.subtract(remainSpeaccount);
        if (remainAmt.compareTo(BigDecimal.ZERO) >= 0) {
            debtBackedModel.setBackedSpeaccount(remainSpeaccount);
        } else {
            debtBackedModel.setBackedSpeaccount(backedAmt);
        }
        return remainAmt;
    }
}
