package com.jqtx.windows.command;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.jqtx.windows.common.config.BaseConfig;
import com.jqtx.windows.common.config.SpringContextConfig;
import com.jqtx.windows.common.exception.BizException;
import com.jqtx.windows.common.factory.AbstractCommand;
import com.jqtx.windows.common.factory.RuntimeContextHolder;
import com.jqtx.windows.component.CreditComponent;
import com.jqtx.windows.component.enums.CreditResultStatusEnum;
import com.jqtx.windows.component.model.CreditModel;
import com.jqtx.windows.infrastructure.enums.ExceptionCodeEnum;
import com.jqtx.windows.web.request.GetCreditRequest;
import com.jqtx.windows.web.response.CreditDetailResponse;
import com.jqtx.windows.web.response.JsonResult;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class GetCreditCommand extends AbstractCommand {
    @Autowired
    private CreditComponent creditComponent;
    @Autowired
    private BaseConfig baseConfig;

    public JsonResult execute() {
        String requestBody = RuntimeContextHolder.currentRuntimeContext().getRequestBody();
        GetCreditRequest request = JSON.parseObject(requestBody, GetCreditRequest.class);
        if (ObjectUtil.isNull(request) || StringUtils.isBlank(request.getOrderNo())) {
            throw new BizException(ExceptionCodeEnum.PARAMS_NULL);
        }
        CreditDetailResponse response = this.convert(request.getOrderNo());
        response.setTransNo(request.getTransNo());
        return JsonResult.success(response);
    }

    public CreditDetailResponse convert(String orderNo) {
        CreditDetailResponse response = new CreditDetailResponse();
        CreditModel credit = creditComponent.getByOrderNo(orderNo);
        if (ObjectUtil.isNull(credit)) {
            throw new BizException(ExceptionCodeEnum.BIZ_CREDIT_NOT_FOUND_ERROR);
        }
        response.setStatus("0");
        response.setApprovalAmount("0");
        response.setOrderNo(credit.getOrderNo());
        response.setUpdateTime(LocalDateTimeUtil.formatNormal(credit.getGmtModified()));
        //偶数TODO
        if (baseConfig.getUseBaffle().equals("use")) {
            response.setStatus("1");
            response.setApprovalAmount("10000.00");
//            String substring = credit.getOrderNo().substring(credit.getOrderNo().length() - 1, credit.getOrderNo().length());
//            if (Integer.valueOf(substring) % 2 == 0) {
//                response.setStatus("1");
//                response.setApprovalAmount("10000.00");
//            } else {
//                response.setStatus("2");
//                //response.setApprovalAmount("10000.00");
//                response.setRemark("挡板拒绝");
//            }
        } else {
            if (CreditResultStatusEnum.PASS.name().equals(credit.getCreditResult())) {
                response.setStatus("1");
                response.setApprovalAmount(credit.getCreditAmt());
            }
            if (CreditResultStatusEnum.REFUSED.name().equals(credit.getCreditResult())) {
                response.setStatus("2");
            }
            response.setRemark(credit.getMemo());
        }
        return response;
    }
}
