package com.jqtx.windows.command;

import com.alibaba.fastjson.JSON;
import com.jqtx.windows.common.config.BaseConfig;
import com.jqtx.windows.common.config.SpringContextConfig;
import com.jqtx.windows.common.exception.BizException;
import com.jqtx.windows.common.factory.AbstractCommand;
import com.jqtx.windows.common.factory.RuntimeContextHolder;
import com.jqtx.windows.component.LoanComponent;
import com.jqtx.windows.component.model.LoanModel;
import com.jqtx.windows.infrastructure.enums.ExceptionCodeEnum;
import com.jqtx.windows.web.response.JsonResult;
import com.jqtx.windows.web.response.LoanDetailResponse;
import com.jqtx.windows.web.response.LoanSubmitResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Objects;

/**
 * 提现审核结果查询
 */
@Service
@Slf4j
public class LoanStatusCommand extends AbstractCommand {

    @Autowired
    private LoanComponent loanComponent;
    @Autowired
    private BaseConfig baseConfig;

    @Override
    public JsonResult execute() {
        String request = RuntimeContextHolder.currentRuntimeContext().getRequestBody();
        LoanSubmitResponse loanSubmitResponse = JSON.parseObject(request, LoanSubmitResponse.class);
        LoanModel loanModel = loanComponent.getByLoanNo(loanSubmitResponse.getLoanNo());

        LoanDetailResponse response = new LoanDetailResponse();
        response.setOrderNo(loanSubmitResponse.getOrderNo());
        response.setStatus(0);
        response.setRemark("提现审核中");
        if (Objects.isNull(loanModel)) {
            throw new BizException(ExceptionCodeEnum.BIZ_CREDIT_NOT_FOUND_ERROR);
        }
        if (baseConfig.getUseBaffle().equals("use")) {
            response.setStatus(1);
            response.setRemark("挡板：提现通过");
//            String substring = loanModel.getOrderNo().substring(loanModel.getOrderNo().length() - 1, loanModel.getOrderNo().length());
//            if (Integer.valueOf(substring) % 2 == 0) {
//                response.setStatus(1);
//                response.setRemark("挡板：提现通过");
//            } else {
//                response.setStatus(2);
//                response.setRemark("挡板：提现拒绝，去别家看看");
//            }
        } else {
            if ("PASS".equals(loanModel.getLoanCreditResult())) {
                response.setStatus(1);
                response.setRemark("提现通过");
            }
            if ("REFUSED".equals(loanModel.getLoanCreditResult())) {
                response.setStatus(2);
                response.setRemark("提现拒绝，去别家看看");
            }
        }
        return JsonResult.success(response);
    }
}
