package com.jqtx.windows.command;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson.JSON;
import com.jqtx.windows.common.exception.BizException;
import com.jqtx.windows.common.factory.AbstractCommand;
import com.jqtx.windows.common.factory.RuntimeContextHolder;
import com.jqtx.windows.component.CreditComponent;
import com.jqtx.windows.component.LoanComponent;
import com.jqtx.windows.component.enums.CreditResultStatusEnum;
import com.jqtx.windows.component.model.CreditModel;
import com.jqtx.windows.component.model.LoanCreditSubmitModel;
import com.jqtx.windows.component.model.LoanModel;
import com.jqtx.windows.infrastructure.enums.ExceptionCodeEnum;
import com.jqtx.windows.infrastructure.feign.IWakandaFeignService;
import com.jqtx.windows.web.response.JsonResult;
import com.jqtx.windows.web.response.LoanSubmitResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Objects;

/**
 * 提现授信审核
 */
@Service
@Slf4j
public class LoanSubmitCommand extends AbstractCommand {

    @Autowired
    private LoanComponent loanComponent;
    @Autowired
    private CreditComponent creditComponent;
    @Autowired
    private IWakandaFeignService wakandaFeignService;

    @Override
    public JsonResult execute() {
        String request = RuntimeContextHolder.currentRuntimeContext().getRequestBody();
        LoanSubmitResponse loanSubmitResponse = JSON.parseObject(request, LoanSubmitResponse.class);
        CreditModel byOrderNo = creditComponent.getByOrderNo(loanSubmitResponse.getOrderNo());
        if (Objects.isNull(byOrderNo)) {
            throw new BizException(ExceptionCodeEnum.BIZ_CREDIT_NOT_FOUND_ERROR);
        }
        // 保存windows_loan表
        LoanModel loanModel = new LoanModel();
        loanModel.setOrderNo(loanSubmitResponse.getOrderNo());
        loanModel.setLoanNo(loanSubmitResponse.getLoanNo());
        loanModel.setLoanStatus(CreditResultStatusEnum.INIT.name());
        loanModel.setGmtCreated(LocalDateTime.now());
        loanModel.setGmtModified(LocalDateTime.now());
        Boolean insert = loanComponent.insert(loanModel);
        if (!insert) {
            throw new BizException(ExceptionCodeEnum.SYSTEM_SQL_INSERT_ERROR);
        }
        // 提现授信
        LoanCreditSubmitModel model = new LoanCreditSubmitModel();
        model.setOrderNo(byOrderNo.getOrderNo());
        JsonResult<Boolean> booleanJsonResult = wakandaFeignService.loanCredit(model);
        return JsonResult.success(booleanJsonResult.isSuccess());
    }
}
