package com.jqtx.windows.command;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.Assert;
import com.alibaba.fastjson.JSON;
import com.jqtx.windows.common.converter.ProtocolConverter;
import com.jqtx.windows.common.factory.AbstractCommand;
import com.jqtx.windows.common.factory.RuntimeContextHolder;
import com.jqtx.windows.common.utils.ValidateUtil;
import com.jqtx.windows.component.LoanComponent;
import com.jqtx.windows.component.LoanPlanComponent;
import com.jqtx.windows.component.enums.LoanStatusEnum;
import com.jqtx.windows.component.model.LoanModel;
import com.jqtx.windows.component.model.LoanPlanModel;
import com.jqtx.windows.component.model.UploadPdfDetailRequest;
import com.jqtx.windows.component.model.UploadPdfRequest;
import com.jqtx.windows.component.seq.BizSeqNoConfig;
import com.jqtx.windows.component.seq.ISeqNoGenerateComponent;
import com.jqtx.windows.infrastructure.feign.IProtocalFeignService;
import com.jqtx.windows.web.request.SyncLoanRequest;
import com.jqtx.windows.web.response.JsonResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;


@Service
public class SyncLoanCommand extends AbstractCommand {

    @Autowired
    private LoanComponent loanComponent;
    @Autowired
    private ISeqNoGenerateComponent iSeqNoGenerateComponent;
    @Autowired
    private LoanPlanComponent loanPlanComponent;
    @Autowired
    private IProtocalFeignService iProtocalFeignService;
    @Autowired
    private ProtocolConverter protocolConverter;
    @Override
    public JsonResult execute() {
        String request = RuntimeContextHolder.currentRuntimeContext().getRequestBody();
        SyncLoanRequest syncLoanRequest = JSON.parseObject(request, SyncLoanRequest.class);
        //校验
        ValidateUtil.valid(syncLoanRequest);
        //修改loan
        LoanModel loanModel = loanComponent.getByLoanNo(syncLoanRequest.getLoanNo());
        Assert.isFalse(loanModel == null, "借款不存在");
        Assert.isFalse(!loanModel.getLoanStatus().equals(LoanStatusEnum.AUDITED.name()), "订单状态错误");
        //新增plan
        BigDecimal loanAmt = syncLoanRequest.getPlanList().stream().map(SyncLoanRequest.LoanPlan::getPlanAmt).map(BigDecimal::new).reduce(BigDecimal.ZERO, BigDecimal::add);
        BigDecimal raiseAmt = syncLoanRequest.getPlanList().stream().map(SyncLoanRequest.LoanPlan::getPlanBen).map(BigDecimal::new).reduce(BigDecimal.ZERO, BigDecimal::add);
        BigDecimal rateAmt = syncLoanRequest.getPlanList().stream().map(SyncLoanRequest.LoanPlan::getPlanXi).map(BigDecimal::new).reduce(BigDecimal.ZERO, BigDecimal::add);
        BigDecimal serviceAmt = syncLoanRequest.getPlanList().stream().map(SyncLoanRequest.LoanPlan::getPlanFu).map(BigDecimal::new).reduce(BigDecimal.ZERO, BigDecimal::add);
        loanModel.setLoanAmt(loanAmt);
        loanModel.setRaiseAmt(raiseAmt);
        loanModel.setServiceAmt(serviceAmt);
        loanModel.setRateAmt(rateAmt);
        loanModel.setBurdenAmt(BigDecimal.ZERO);
        loanModel.setLoanUsage(syncLoanRequest.getLoanUsage());
        loanModel.setRaiseDate(LocalDateTime.parse(syncLoanRequest.getRaiseDate()));
        loanModel.setLoanStatus(LoanStatusEnum.BACKING.name());
        loanModel.setBankAccount(syncLoanRequest.getBankAccount());
        loanModel.setLoanTerm(syncLoanRequest.getLoanTerm());
        loanModel.setGmtModified(LocalDateTime.now());
        int i = loanComponent.updateByLoanNo(loanModel);
        //新增plan
        List<SyncLoanRequest.LoanPlan> planList = syncLoanRequest.getPlanList();
        planList.forEach(loanPlan -> {
            String planNo = iSeqNoGenerateComponent.genCommonSeqNo(BizSeqNoConfig.PLAN_NO_KEY, BizSeqNoConfig.PLAN_NO_PREFIX);
            LoanPlanModel loanPlanModel = BeanUtil.toBean(loanPlan, LoanPlanModel.class);
            loanPlanModel.setPlanNo(planNo);
            //新增
            loanPlanComponent.insert(loanPlanModel);
        });
        //ok
        UploadPdfRequest uploadPdfRequest=new UploadPdfRequest();
        uploadPdfRequest.setLoanNo(syncLoanRequest.getLoanNo());
        uploadPdfRequest.setOrderNo(syncLoanRequest.getOrderNo());
        List<UploadPdfDetailRequest> detailRequestList=   syncLoanRequest.getProtocolList().stream().map(protocolConverter::toModel).collect(Collectors.toList());
        uploadPdfRequest.setDetailRequestList(detailRequestList);
        iProtocalFeignService.uploadProtocolPdf(uploadPdfRequest);
        return JsonResult.success();
    }
}
