package com.jqtx.windows.common.annotiation;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;

@Aspect
@Slf4j
@Configuration
public class BaseLogAop {

    public BaseLogAop(){
    }

    @Pointcut("@annotation(com.jqtx.windows.common.annotiation.BaseLogAnnotated)")
    public void logAspect() {
    }

    @Before("logAspect()")
    private void before(JoinPoint point){
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        Object[] args = point.getArgs();
        if(ObjectUtil.isNotNull(args) || point.getArgs().length!=0){
            log.info("[接口:{},参数:----------->{}]",request.getRequestURI(),JSON.toJSONString(args));
        }else{
            log.info("[接口:{},参数:----------->{}]",request.getRequestURI(),null);
        }
    }

    @AfterReturning(value = "logAspect() && @annotation(commonLogAnnotated)", returning = "response")
    private void after(JoinPoint point, com.jqtx.windows.common.annotiation.BaseLogAnnotated commonLogAnnotated, Object response){
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        log.info("[接口:{},结果:===========>{}]",request.getRequestURI(),JSON.toJSONString(response));
    }
}
