package com.jqtx.windows.common.config.snow;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.net.UnknownHostException;

/**
 * 雪花算法唯一序号生成工具
 *
 * @version $Id: SnowSequenceHelper.java, v 0.1 2018年3月28日 下午4:46:41 zhoulongsheng Exp $
 */
@Component
public class SnowSequenceHelper {

    public final static String USER_ID = "16450";
    public final static String PRE_CREDIT_NO = "16750";
    public final static String CREDIT_NO = "16760";
    public final static String ACTION_NO = "0000";

    public final static String CREDIT_ACTION_NO = "1111";
    private static SnowFlake instance = null;

    private static final Logger logger = LoggerFactory.getLogger(SnowSequenceHelper.class);

    /**
     * 生成规则：16位长度雪花算法序列号
     *
     * @return
     */
    public static long nextSequence() {
        return instance.nextId();
    }

    /**
     * 生成规则：4位前缀编号 + 16位雪花算法序列号
     * <p>
     * 前缀编号:@link SnowSequenceHelper.PREFIX_*
     *
     * @param prefix 指定前缀
     * @return
     */
    public static String nextSequence(String prefix) {
        return prefix + instance.nextId();
    }

    /**
     * 初始化雪花算法序列号生成器
     *
     * @throws UnknownHostException
     */
    @PostConstruct
    private synchronized void init() throws UnknownHostException {
        if (instance != null) {
            logger.info("------------- Snow Flake Sequence has initialized --------------");
            return;
        }

        try {
            String[] ipArray = GrabMachineIpUtil.getLocalHostLANAddress().getHostAddress().split("\\.");
            long addr_t = Long.valueOf(ipArray[2]).longValue();
            long addr_f = Long.valueOf(ipArray[3]).longValue();

            long machineId = getScopeId(addr_t, SnowFlake.MAX_MACHINE_NUM);
            long dataCenterId = getScopeId(addr_f, SnowFlake.MAX_DATA_CENTER_NUM);

            instance = new SnowFlake(dataCenterId, machineId);
            logger.info("------------- Snow Flake Sequence initialize success --------------, dataCenterId:{}, machineId:{}", dataCenterId, machineId);

        } catch (UnknownHostException ue) {
            logger.error("------------- Snow Flake Sequence initialize failed --------------", ue);
            throw ue;
        }
    }

    private static long getScopeId(long addr, long maxId) throws UnknownHostException {
        long r = addr;
        if (addr >= 0L && addr <= maxId) {
            return addr;
        } else {
            do {
                do {
                    r >>= 2;
                } while (r < 0L);
            } while (r > maxId);

            return r;
        }
    }

}
