package com.jqtx.windows.common.factory;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.jqtx.windows.common.exception.BizException;
import com.jqtx.windows.infrastructure.enums.ExceptionCodeEnum;
import com.jqtx.windows.utils.AbcRsaUtil;
import com.jqtx.windows.web.config.RouterConfig;
import com.jqtx.windows.web.response.JsonResult;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringEscapeUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

@Service
@Slf4j
public class CommandProxy {

    @Value("${abc.publicKey}")
    private String publicKey;

    @Value("${abc.yy_privateKey}")
    private String YyPpivateKey;


    public JsonResult test(String request,String method){
        AbstractCommand command = RouterConfig.getCommand(method);
        if(ObjectUtil.isNull(command)){
            return JsonResult.error(ExceptionCodeEnum.SYSTEM_URL_ERROR);
        }
        ChannelConfig channelConfig = new ChannelConfig();
        channelConfig.setAppId("9999999");
        this.setLocalContext(request,channelConfig);
        log.info("[浙农请求request:处理:{}------------>{}]",command.getClass().getSimpleName(),request);
        JsonResult result = command.doAction();
        log.info("[浙农请求response:处理:{}=============>{}]",command.getClass().getSimpleName(),JSON.toJSONString(result));
        return result;
    }

    public JsonResult call(String request,String method){
        AbstractCommand command = RouterConfig.getCommand(method);
        if(ObjectUtil.isNull(command)){
            return JsonResult.error(ExceptionCodeEnum.SYSTEM_URL_ERROR);
        }
        JSONObject jsonObject = JSONObject.parseObject(request);
        String requestBody = jsonObject.getString("content");
        String sign = jsonObject.getString("sign");
        String appId = jsonObject.getString("appId");
        String decodeBody = AbcRsaUtil.decodeParam(requestBody, sign, YyPpivateKey, publicKey);
        if(StringUtils.isBlank(decodeBody)){
            return JsonResult.error(ExceptionCodeEnum.SYSTEM_AUTH_ERROR);
        }
        ChannelConfig channelConfig = new ChannelConfig();
        channelConfig.setAppId(appId);
        this.setLocalContext(decodeBody,channelConfig);
        log.info("[浙农请求request:处理:{}------------>{}]",command.getClass().getSimpleName(),decodeBody);
        JsonResult result = command.doAction();
        log.info("[浙农请求response:处理:{}=============>{}]",command.getClass().getSimpleName(),JSON.toJSONString(result));
        return result;
    }

    public void setLocalContext(String requestBody, ChannelConfig channelConfig) {
        RuntimeContext context = RuntimeContextHolder.currentRuntimeContext();
        if (context == null) {
            context = new RuntimeContext();
        }
        context.setRequestBody(requestBody);
        context.setChannelConfig(channelConfig);
        RuntimeContextHolder.setCurrentRuntimeContext(context);
    }
}
