package com.jqtx.windows.web.response;

import com.jqtx.windows.common.exception.BizException;
import com.jqtx.windows.infrastructure.enums.ExceptionCodeEnum;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.ToString;

import java.io.Serializable;

@ApiModel(
        value = "JsonResult",
        description = "通用返回结果模型"
)
@ToString
public class JsonResult<T> implements Serializable {
    private static final long serialVersionUID = 1475348231900998033L;
    @ApiModelProperty("状态码")
    private String code;
    @ApiModelProperty("请求成功与否,true:成功,false:失败")
    private Boolean success;
    @ApiModelProperty("提示信息")
    private String message;
    @ApiModelProperty("返回业务数据")
    private T result;

    public JsonResult() {
    }

    public JsonResult(boolean isSuccess, String code, String message) {
        this.success = isSuccess;
        this.code = code;
        this.message = message;
    }

    public static JsonResult success() {
        return new JsonResult(true, "200", "操作成功!");
    }

    public static JsonResult success(Object data) {
        JsonResult result = new JsonResult(true, "200", "操作成功!");
        result.setResult(data);
        return result;
    }

    public static JsonResult error(String message) {
        JsonResult result = new JsonResult(false, "500", message);
        return result;
    }

    public static JsonResult error(ExceptionCodeEnum message) {
        JsonResult result = new JsonResult(false, message.getCode(), message.getMessageCN());
        return result;
    }

    public static JsonResult error(ExceptionCodeEnum message, String msg) {
        JsonResult result = new JsonResult(false, message.getCode(), msg);
        return result;
    }

    public static JsonResult error(String message, String code) {
        JsonResult result = new JsonResult(false, code, message);
        result.setCode(code);
        return result;
    }

    public void apiCheck() {
        if (!this.isSuccess()) {
            throw new BizException(ExceptionCodeEnum.SYSTEM_REMOTE_ERROR, message);
        }
    }

    public void resultCheck() {
        if (!this.isSuccess()) {
            throw new BizException(ExceptionCodeEnum.SYSTEM_REMOTE_ERROR, message);
        }
        if (this.getResult() == null) {
            throw new BizException(ExceptionCodeEnum.SYSTEM_REMOTE_ERROR, message);
        }
    }

    public String getCode() {
        return this.code;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public boolean isSuccess() {
        return this.success;
    }

    public void setSuccess(boolean success) {
        this.success = success;
    }

    public String getMessage() {
        return this.message;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public T getResult() {
        return this.result;
    }

    public void setResult(T result) {
        this.result = result;
    }

}
